/*
 * Smell-o-scope project
 */

#include <ESP8266WiFi.h>
#include <DNSServer.h>
#include <ESP8266WebServer.h>

// array sizes
#define LGRAPH 60
#define LFILTER 14

// pins for the multiplexer
#define S0 4
#define S1 0
#define S2 2

int sensorPin = 0;    // select the input pin for the potentiometer
short graph1[LGRAPH];
short filter1[LFILTER];
short max1 = 0;
short min1 = 1000;
short graph2[LGRAPH];
short filter2[LFILTER];
short max2 = 0;
short min2 = 1000;
short currGraph = 0;
short currFilter = 0;
short filterSum = 0;
short value;
unsigned long prevTime;
const unsigned int timeInterval = 2400; //ms

const byte DNS_PORT = 53;
IPAddress apIP(192, 168, 10, 100);
DNSServer dnsServer;
ESP8266WebServer webServer(80);

String prepareHTML(){
  String responseHTML = "<!DOCTYPE html><html><head><meta http-equiv=\"Content-type\" content=\"text/html; charset=utf-8\" /> <title>Smell-o-scope</title> <style> * { margin: 0; padding: 0; font-family: \"Segoe UI\", Tahoma, Geneva, Verdana, sans-serif; } .wrapper { width: 600px; margin: 1.8em auto; text-align: center; } .text-wrapper { width: 600px; margin: 0em auto 1.8em; text-align: center; } .bottom-wrapper { width: 600px; margin: 1.8em auto 0.2em; text-align: center; } .box { padding: 1.8em; outline: 1px solid #aaa; border-radius: 5px; } .graph-box { width: 100%; height: 300px; background: white; } .point { position: relative; width: 10px; height: 300px; float: left; } .point-2 { position: absolute; bottom: 0; width: 9px; background: lightblue; -webkit-border-top-left-radius: 30px; -webkit-border-top-right-radius: 30px; } .point-2 span { display: block; position: absolute; top: -20px; font-family: \"Segoe UI\", Tahoma, Geneva, Verdana, sans-serif; font-weight: bolder; font-size: 9px; color: #333; left: 0; transform: rotate(-90deg); padding-bottom: 0.1em; } .point:hover .point-2 { background: darkblue; } .threshold { background: red; } </style>"
                        "<script> const xhr = new XMLHttpRequest(); xhr.onload = () => { const data = JSON.parse(xhr.responseText); document.querySelector(\"#min1\").innerHTML = data[\"min1\"]; document.querySelector(\"#max1\").innerHTML = data[\"max1\"]; document.querySelector(\"#min2\").innerHTML = data[\"min2\"]; document.querySelector(\"#max2\").innerHTML = data[\"max2\"]; for (var index = 0; index < data[\"sensor1\"].length; index++) { if(data[\"sensor1\"][data[\"sensor1\"].length-1-index]>100){document.querySelector(\"#i1\" + index).style.backgroundColor = 'red';} else {document.querySelector(\"#i1\" + index).style.backgroundColor = 'lightblue';} document.querySelector(\"#i1\" + index).style.height = data[\"sensor1\"][data[\"sensor1\"].length-1-index] + \"%\"; document.querySelector(\"#i1\" + index).innerHTML = \"<span>\" + data[\"sensor1\"][data[\"sensor1\"].length-1-index] + \"</span>\"; } for (var index = 0; index < data[\"sensor2\"].length; index++) { if(data[\"sensor2\"][data[\"sensor2\"].length-1-index]>400){document.querySelector(\"#i2\" + index).style.backgroundColor = 'red';} else {document.querySelector(\"#i2\" + index).style.backgroundColor = 'lightblue';} document.querySelector(\"#i2\" + index).style.height = Math.floor(data[\"sensor2\"][data[\"sensor2\"].length-1-index]/8) + \"%\"; document.querySelector(\"#i2\" + index).innerHTML = \"<span>\" + data[\"sensor2\"][data[\"sensor2\"].length-1-index] + \"</span>\"; } }; window.setInterval(function () { xhr.open(\"GET\", \"http://192.168.10.100/data\"); xhr.send(); }, 5000); </script>"
                        "</head> <body> <div class=\"wrapper\"> <h1>Exhaust gas analysis system</h1> <h2>Smell-o-scope</h2> </div> <div class=\"wrapper box\"> <div class=\"text-wrapper\"> <h2>Carbon monoxide, CH<sub>4</sub> and LPG sensor (MQ-9)</h2> <h3>Last 5 minutes, 5s step; ppm</h3> </div> <div class=\"graph-box\">";

  for (int i = 0; i < LGRAPH; i++){
    responseHTML += "<div class=\"point\"><div class=\"point-2\" id=\"i1"  + String(i) + "\"><span></span></div></div>";
  }

  responseHTML += "<div style=\"clear: both\"></div> </div> <div class=\"bottom-wrapper\"> <p>Rolling values for the last 24h:</p> <p>Maximum: <span id=\"max1\"></span>;</p> <p>Minimum: <span id=\"min1\"></span>;</p> </div> </div> <div class=\"wrapper box\"> <div class=\"text-wrapper\"> <h2>Alcohol gas sensor (MQ-3)</h2> <h3>Last 5 minutes, 5s step; ppm</h3> </div> <div class=\"graph-box\">";

  for (int i = 0; i < LGRAPH; i++){
    responseHTML += "<div class=\"point\"><div class=\"point-2\" id=\"i2"  + String(i) + "\"><span></span></div></div>";
  }
  responseHTML = responseHTML + " <div style=\"clear: both\"></div> </div> <div class=\"bottom-wrapper\"> <p>Rolling values for the last 24h:</p> <p>Maximum: <span id=\"max2\"></span>;</p> <p>Minimum: <span id=\"min2\"></span>;</p> </div> </div> </body></html>";
  return responseHTML;
}

String prepareJSON(){
  String responseJSON = "{\"sensor1\":[";
  for (int i = currGraph; i < LGRAPH; i++){
    responseJSON += String(graph1[i]) + ",";
  }
  for (int i = 0; i < currGraph; i++){
    responseJSON += String(graph1[i]) + ",";
  }
  responseJSON = responseJSON.substring(0, responseJSON.length() - 1);
  responseJSON += "], \"sensor2\":[";
  for (int i = currGraph; i < LGRAPH; i++){
    responseJSON += String(graph2[i]) + ",";
  }
  for (int i = 0; i < currGraph; i++){
    responseJSON += String(graph2[i]) + ",";
  }
  responseJSON = responseJSON.substring(0, responseJSON.length() - 1);
  responseJSON += "],";
  responseJSON += "\"min1\":"+String(min1)+", ";
  responseJSON += "\"min2\":"+String(min2)+", ";
  responseJSON += "\"max1\":"+String(max1)+", ";
  responseJSON += "\"max2\":"+String(max2)+"}";
  return responseJSON;
}

void setMultiplexer( int i ){
  int s0Value = i & 0x01;      // get value of first bit
  int s1Value = (i>>1) & 0x01; // get value of second bit
  int s2Value = (i>>2) & 0x01; // get value of third bit

  digitalWrite( S0, s0Value ); // turn first pin on or off
  digitalWrite( S1, s1Value ); // turn second pin on or off
  digitalWrite( S2, s2Value ); // turn third pin on or off
}

void setup() {
  // Set multiplexer controller pins to output
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);

  // initializing arrays
  for (int i = 0; i < LFILTER; i++){
    filter1[i] = 0;
  }
  for (int i = 0; i < LGRAPH; i++){
    graph1[i] = 0;
  }
  for (int i = 0; i < LFILTER; i++){
    filter2[i] = 0;
  }
  for (int i = 0; i < LGRAPH; i++){
    graph2[i] = 0;
  }
  
  // Set Wi-Fi
  WiFi.mode(WIFI_AP);
  WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));
  WiFi.softAP("Smell-o-scope");
  delay(10);
  // if DNSServer is started with "*" for domain name, it will reply with
  // provided IP to all DNS request
  dnsServer.start(DNS_PORT, "*", apIP);
  
  // reply to all requests with same HTML
  webServer.onNotFound([]() {
    webServer.send(200, "text/html", prepareHTML());
  });

  webServer.on("/", [](){
    webServer.send(200, "text/html", prepareHTML());
  });

  webServer.on("/data", [](){
    webServer.send(200, "text/json", prepareJSON());
  });
  
  webServer.begin();
  delay(10);
  prevTime = millis();
}

void loop() {
  setMultiplexer(0);
  delay(5);
  filter1[currFilter] = analogRead(sensorPin);
  delay(5);
  setMultiplexer(1);
  delay(5);
  filter2[currFilter] = analogRead(sensorPin);
  delay(5);
  
  currFilter += 1;
  if (currFilter >= LFILTER){
    currFilter = 0;
  }
  
  if ((millis() - prevTime) >= timeInterval){
    // graph 1 prepare
    filterSum = 0;
    for (int i = 0; i < LFILTER; i++){
      filterSum += filter1[i];
    }
    
    value = div(filterSum, LFILTER).quot;
    graph1[currGraph] = value;
    if (value < min1){
      min1 = value;
    }
    if (value > max1){
      max1 = value;
    }
    
    // graph 2 prepare
    filterSum = 0;
    for (int i = 0; i < LFILTER; i++){
      filterSum += filter2[i];
    }

    value = div(filterSum, LFILTER).quot;
    graph2[currGraph] = value;
    if (value < min2){
      min2 = value;
    }
    if (value > max2){
      max2 = value;
    }
    
    prevTime = millis();
    currGraph += 1;
    if (currGraph >= LGRAPH){
      currGraph = 0;
    }
    webServer.on("/data", [](){
      webServer.send(200, "text/json", prepareJSON());
    });
    
  }
  
  dnsServer.processNextRequest();
  webServer.handleClient();
}
